using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Input;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using SIS.Contracts.PersonListModul;
using SIS.Contracts.PersonService;
using SIS.Model;

namespace SIS.PersonListModul
{
    public class PersonListViewModel : IPersonListViewModel, INotifyPropertyChanged
    {
        private readonly IPersonService _personService;
        private readonly IEventAggregator _eventAggregator;

        /// <summary>
        /// Initializes a new instance of the <see cref="PersonListViewModel"/> class.
        /// </summary>
        /// <param name="personService">The person service.</param>
        public PersonListViewModel(IPersonService personService, IEventAggregator eventAggregator)
        {
            _personService = personService;
            _eventAggregator = eventAggregator;
            PersonsCollection = new ObservableCollection<Person>();
            GetPersonsCommand = new DelegateCommand<object>(getPersons);
        }

        private ObservableCollection<Person> _personsCollection;

        /// <summary>
        /// Gets or sets the persons collection.
        /// </summary>
        /// <value>The persons collection.</value>
        public ObservableCollection<Person> PersonsCollection
        {
            get { return _personsCollection; }
            private set
            {
                _personsCollection = value;
                OnPropertyChanged("PersonsCollection");
            }
        }

        private Person _selectedPerson;

        /// <summary>
        /// Gets or sets the selected person.
        /// </summary>
        /// <value>The selected person.</value>
        public Person SelectedPerson
        {
            get { return _selectedPerson; }
            set
            {
                _selectedPerson = value;
                OnPropertyChanged("SelectedPerson");
                _eventAggregator.GetEvent<SIS.Infrastructure.PersonSelectionChangedEvent>().Publish(_selectedPerson);
            }
        }

        /// <summary>
        /// Gets or sets the get persons command.
        /// </summary>
        /// <value>The get persons command.</value>
        public ICommand GetPersonsCommand
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the person selected command.
        /// </summary>
        /// <value>The person selected command.</value>
        public ICommand PersonSelectedCommand
        {
            get;
            private set;
        }


        /// <summary>
        /// Gets the persons.
        /// </summary>
        /// <param name="o">The o.</param>
        private void getPersons(object o)
        {
            var persons = _personService.GetPersons();
            PersonsCollection = new ObservableCollection<Person>(persons);

        }


        #region NotifyPropertyChanged
        [Conditional("DEBUG")]
        [DebuggerStepThrough]
        public void VerifyPropertyName(string propertyName)
        {
            // Verify that the property name matches a real,  
            // public, instance property on this object.
            if (TypeDescriptor.GetProperties(this)[propertyName] == null)
            {
                string msg = "Invalid property name: " + propertyName;
                Debug.Fail(msg);
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            this.VerifyPropertyName(propertyName);

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }
        #endregion

    }
}